package com.android.sample;

import android.app.Application;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import androidx.localbroadcastmanager.content.LocalBroadcastManager;

import android.text.TextUtils;
import android.util.Log;
import android.widget.Toast;

import static com.usdk.android.UsdkThreeDS2ServiceImpl.USDK_LOST_STATE_ACTION;
import static com.usdk.android.UsdkThreeDS2ServiceImpl.USDK_THREE_DS_SERVER_TRANS_ID_KEY;

import io.sentry.android.core.SentryAndroid;

public class SampleApplication extends Application {

    private static final String TAG = SampleApplication.class.getSimpleName();

    private Settings settings;

    @Override
    public void onCreate() {
        super.onCreate();

        settings = new Settings(getApplicationContext());

        if (!TextUtils.isEmpty(settings.getSentryDsn())) {
            SentryAndroid.init(this, options -> {
                options.setDsn(settings.getSentryDsn());
            });
        }


        BroadcastReceiver receiver = new BroadcastReceiver() {
            @Override
            public void onReceive(Context context, Intent intent) {
                if (USDK_LOST_STATE_ACTION.equals(intent.getAction())) {
                    StringBuilder message = new StringBuilder("State of the app has been lost.");
                    if (intent.hasExtra(USDK_THREE_DS_SERVER_TRANS_ID_KEY)) {
                        String threeDSServerTransID = intent.getStringExtra(USDK_THREE_DS_SERVER_TRANS_ID_KEY);
                        Log.d(TAG, "Three DS Server Transaction ID: " + threeDSServerTransID);
                        message.append(" Transaction ID: ").append(threeDSServerTransID);
                    }
                    Toast.makeText(getApplicationContext(), message, Toast.LENGTH_LONG).show();
                }
            }
        };

        LocalBroadcastManager.getInstance(this).registerReceiver(receiver, new IntentFilter(USDK_LOST_STATE_ACTION));
    }
}
