package com.android.sample;

import android.content.Context;
import android.content.SharedPreferences;

import com.android.sample.enums.ApiInterface;

public class Settings {

    private static final String PREFS_NAME = "uSDK_Sample_App";

    private static final String EMAIL_KEY = "email";
    private static final String PASSWORD_KEY = "password";
    private static final String BACKEND_URL_KEY = "backend_url";
    private static final String LICENSE_KEY_KEY = "license_key";
    private static final String ACCOUNT_NUMBER_KEY = "account_number";
    private static final String API_INTERFACE_KEY = "api_interface";
    private static final String SENTRY_DSN_KEY = "sentry_dsn";

    private final SharedPreferences sharedPreferences;
    private final Context context;

    public Settings(Context context) {
        this.sharedPreferences = context.getSharedPreferences(PREFS_NAME, Context.MODE_PRIVATE);
        this.context = context.getApplicationContext();
    }

    public void saveEmail(String email) {
        sharedPreferences.edit().putString(EMAIL_KEY, email).apply();
    }

    public String getEmail() {
        return sharedPreferences.getString(EMAIL_KEY, "");
    }

    public void savePassword(String password) {
        sharedPreferences.edit().putString(PASSWORD_KEY, password).apply();
    }

    public String getPassword() {
        return sharedPreferences.getString(PASSWORD_KEY, "");
    }

    public void saveBackendUrl(String backendUrl) {
        sharedPreferences.edit().putString(BACKEND_URL_KEY, backendUrl).apply();
    }

    public String getBackendUrl() {
        return sharedPreferences.getString(BACKEND_URL_KEY, BuildConfig.BACKEND_URL);
    }

    public void saveLicenseKey(String licenseKey) {
        sharedPreferences.edit().putString(LICENSE_KEY_KEY, licenseKey).apply();
    }

    public String getLicenseKey() {
        return sharedPreferences.getString(LICENSE_KEY_KEY, "");
    }

    public void saveAccountNumber(String accountNumber) {
        sharedPreferences.edit().putString(ACCOUNT_NUMBER_KEY, accountNumber).apply();
    }

    public String getAccountNumber() {
        return sharedPreferences.getString(ACCOUNT_NUMBER_KEY, context.getString(R.string.text_default_card_number));
    }

    public void saveApiInterface(ApiInterface apiInterface) {
        sharedPreferences.edit().putString(API_INTERFACE_KEY, apiInterface.name()).apply();
    }

    public ApiInterface getApiInterface() {
        String apiInterfaceName = sharedPreferences.getString(API_INTERFACE_KEY, null);
        if (apiInterfaceName != null) {
            return ApiInterface.valueOf(apiInterfaceName);
        }
        return null;
    }

    public void saveSentryDsn(String sentryDsn) {
        sharedPreferences.edit().putString(SENTRY_DSN_KEY, sentryDsn).apply();
    }

    public String getSentryDsn() {
        return sharedPreferences.getString(SENTRY_DSN_KEY, "");
    }
}
