package com.android.sample.models;

import com.google.gson.Gson;

import org.emvco.threeds.core.AuthenticationRequestParameters;

import java.util.Arrays;
import java.util.List;

public class PurchaseRequest {

    private String sdkMaxTimeout = "60";

    private String acctNumber;

    private DeviceRenderOptions deviceRenderOptions = new DeviceRenderOptions("03", Arrays.asList("01", "02", "03", "04", "05"));

    private AuthRequestParameters authRequestParameters;

    public PurchaseRequest(AuthenticationRequestParameters authParameters, String acctNumber) {
        this.authRequestParameters = new AuthRequestParameters(authParameters);
        this.acctNumber = acctNumber;
    }

    public String toJson() {
        return new Gson().toJson(this);
    }

    String getSdkMaxTimeout() {
        return sdkMaxTimeout;
    }

    public String getAcctNumber() {
        return acctNumber;
    }

    DeviceRenderOptions getDeviceRenderOptions() {
        return deviceRenderOptions;
    }

    AuthRequestParameters getAuthRequestParameters() {
        return authRequestParameters;
    }

    static class DeviceRenderOptions {

        private String sdkInterface;
        private List<String> sdkUiType;

        DeviceRenderOptions(String sdkInterface, List<String> sdkUiType) {
            this.sdkInterface = sdkInterface;
            this.sdkUiType = sdkUiType;
        }
    }

    public static class AuthRequestParameters {

        private String deviceData;
        private String sdkTransactionID;
        private String sdkAppID;
        private String sdkReferenceNumber;
        private String sdkEphemeralPublicKey;
        private String messageVersion;

        public AuthRequestParameters(AuthenticationRequestParameters authParameters) {
            this.deviceData = authParameters.getDeviceData();
            this.sdkTransactionID = authParameters.getSDKTransactionID();
            this.sdkAppID = authParameters.getSDKAppID();
            this.sdkReferenceNumber = authParameters.getSDKReferenceNumber();
            this.sdkEphemeralPublicKey = authParameters.getSDKEphemeralPublicKey();
            this.messageVersion = authParameters.getMessageVersion();
        }

        public String getDeviceData() {
            return deviceData;
        }

        public String getSdkTransactionID() {
            return sdkTransactionID;
        }

        public String getSdkAppID() {
            return sdkAppID;
        }

        public String getSdkReferenceNumber() {
            return sdkReferenceNumber;
        }

        public String getSdkEphemeralPublicKey() {
            return sdkEphemeralPublicKey;
        }

        public String getMessageVersion() {
            return messageVersion;
        }
    }
}
