package com.android.sample.usdk;

import android.app.ProgressDialog;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.os.Handler;
import android.os.Looper;
import androidx.localbroadcastmanager.content.LocalBroadcastManager;
import androidx.core.os.ConfigurationCompat;
import android.util.Log;

import com.android.sample.MainActivity;
import com.android.sample.R;
import com.android.sample.models.PurchaseRequest;
import com.android.sample.models.PurchaseResponse;
import com.android.sample.models.DirectoryServerItem;
import com.android.sample.models.PurchaseData;
import com.android.sample.models.SupportedVersionsResponse;
import com.google.gson.Gson;

import org.emvco.threeds.core.AuthenticationRequestParameters;
import org.emvco.threeds.core.ChallengeParameters;
import org.emvco.threeds.core.ChallengeStatusReceiver;
import org.emvco.threeds.core.CompletionEvent;
import org.emvco.threeds.core.ConfigParameters;
import org.emvco.threeds.core.DirectoryServer;
import org.emvco.threeds.core.ProtocolErrorEvent;
import org.emvco.threeds.core.RuntimeErrorEvent;
import org.emvco.threeds.core.Transaction;
import org.emvco.threeds.core.Warning;
import org.emvco.threeds.core.ui.UiCustomization;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import okhttp3.Call;
import okhttp3.Callback;
import okhttp3.MediaType;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.RequestBody;
import okhttp3.Response;

import static com.android.sample.models.PurchaseResponse.CHALLENGE_TRANS_STATUS;
import static com.usdk.android.UsdkThreeDS2ServiceImpl.INITIALIZATION_ACTION_EXTRA_ERROR_CODE;
import static com.usdk.android.UsdkThreeDS2ServiceImpl.INITIALIZATION_ACTION_EXTRA_ERROR_TYPE;
import static com.usdk.android.UsdkThreeDS2ServiceImpl.INITIALIZATION_ACTION_EXTRA_SUCCESS;
import static com.usdk.android.UsdkThreeDS2ServiceImpl.INTENT_INITIALIZATION_ACTION;
import static com.usdk.android.UsdkThreeDS2ServiceImpl.LICENSE_PARAMETER_KEY;
import static com.usdk.android.UsdkThreeDS2ServiceImpl.USDK_CONFIG_PARAMETER_GROUP;

public class PureInterfaceUsdkManager extends UsdkManager {

    private static final String TAG = SimplifiedInterfaceUsdkManager.class.getSimpleName();

    static final MediaType JSON = MediaType.parse("application/json; charset=utf-8");

    private ProgressDialog progress;
    private final OkHttpClient client = new OkHttpClient();

    public PureInterfaceUsdkManager(MainActivity activity, String licenseKey) {
        super(activity, licenseKey);
    }

    public void initializeUsdk(final InitializeUsdkResultCallback callback) {
        final ProgressDialog progress = new ProgressDialog(activity);
        progress.setMessage(activity.getString(R.string.progress_usdk_init));
        progress.setIndeterminate(true);
        progress.setCancelable(false);
        progress.show();

        BroadcastReceiver receiver = new BroadcastReceiver() {
            @Override
            public void onReceive(Context context, Intent intent) {
                boolean success = intent.getBooleanExtra(INITIALIZATION_ACTION_EXTRA_SUCCESS, false);

                if (success) {
                    progress.dismiss();

                    Log.d(TAG, "3DS SDK initialized successfully");
                    for (Warning warning : threeDS2Service.getWarnings()) {
                        Log.w(TAG, String.format("Detected 3DS warning with ID %s and message: %s", warning.getID(), warning.getMessage()));
                    }
                    callback.onSuccess();
                } else {
                    progress.dismiss();
                    Log.e(TAG, "uSDK initialization failed");

                    String errorCode = intent.getStringExtra(INITIALIZATION_ACTION_EXTRA_ERROR_CODE);
                    String description = intent.getStringExtra(INITIALIZATION_ACTION_EXTRA_ERROR_TYPE);
                    callback.onError(String.format("3DS SDK initialization failed. Error code: %s. Error description: %s", errorCode, description));
                }

                LocalBroadcastManager.getInstance(context).unregisterReceiver(this);
            }
        };

        LocalBroadcastManager.getInstance(activity).registerReceiver(receiver, new IntentFilter(INTENT_INITIALIZATION_ACTION));

        UiCustomization uiCustomization = new UiCustomization();
        uiCustomization.setTheme(R.style.AppTheme);

//        LabelCustomization labelCustomization = new LabelCustomization();
//        labelCustomization.setHeadingTextFontSize(26);
//        labelCustomization.setTextColor("#007987");
//        uiCustomization.setLabelCustomization(labelCustomization);
//
//        ButtonCustomization buttonCustomization = new ButtonCustomization();
//        buttonCustomization.setBackgroundColor("#55bab7");
//        uiCustomization.setButtonCustomization(buttonCustomization, ButtonType.SUBMIT);

        threeDS2Service.initialize(activity, getConfigParameters(), ConfigurationCompat.getLocales(activity.getResources().getConfiguration()).get(0).toString(), uiCustomization);
    }

    public void purchase(final PurchaseData purchaseData) {
        Log.d(TAG, "Purchase using pure interface");
        progress = new ProgressDialog(activity);
        progress.setMessage(activity.getString(R.string.progress_get_supported_versions_by_ds));
        progress.setIndeterminate(true);
        progress.setCancelable(false);
        progress.show();

        getSupportedVersionsByDsRequest(purchaseData, new Callback() {
            @Override
            public void onFailure(Call call, final IOException e) {
                progress.dismiss();
                Log.e(TAG, "Failed supported versions by DS request", e);

                activity.showDialog("Failed to send the request: " + e.getMessage(), "OK");
            }

            @Override
            public void onResponse(Call call, Response response) throws IOException {
                progress.dismiss();
                String responseBody = response.body().string();
                Log.d(TAG, "Supported versions by DS response: " + responseBody);
                if (response.isSuccessful()) {
                    SupportedVersionsResponse parsedResponse = new Gson().fromJson(responseBody, SupportedVersionsResponse.class);
                    List<DirectoryServerItem> directoryServers = parsedResponse.getDirectoryServers();
                    if (!directoryServers.isEmpty()) {
                        DirectoryServerItem ds = directoryServers.get(0);
                        purchaseWithDeterminedDs(purchaseData, ds, parsedResponse.getExtraAuthParams(), directoryServers.size() > 1);
                    } else {
                        activity.showDialog("There is no DS to continue", "OK");
                    }
                } else {
                    activity.showDialog("Invalid supported versions by DS response: " + responseBody, "OK");
                }
            }
        });
    }

    private void purchaseWithDeterminedDs(final PurchaseData purchaseData, DirectoryServerItem directoryServerItem, Map<String, Object> extraAuthParams, boolean dualBrandedCard) {
        final Transaction transaction = threeDS2Service.createTransaction(directoryServerItem.getDsIdentifier(), directoryServerItem.getMessageVersion());

        new Handler(Looper.getMainLooper()).post(new Runnable() {
            @Override
            public void run() {
                progress = transaction.getProgressView(activity);
                progress.show();
            }
        });

        AuthenticationRequestParameters authParams = transaction.getAuthenticationRequestParameters();
        PurchaseRequest authenticationRequest = new PurchaseRequest(authParams, purchaseData.getAccountNumber());
        Map<String, Object> purchaseRequest = generateAuthenticationRequest(purchaseData, authParams, directoryServerItem, extraAuthParams, dualBrandedCard);

        Log.d(TAG, "Auth request: " + authenticationRequest.toJson());
        final Request request = new Request.Builder()
                .url(purchaseData.getBackendUrl() + AUTH_PATH)
                .header("Authorization", String.format("Bearer %s:%s", purchaseData.getEmail(), purchaseData.getPassword()))
                .post(RequestBody.create(JSON, new Gson().toJson(purchaseRequest)))
                .build();

        client.newCall(request).enqueue(new Callback() {
            @Override
            public void onFailure(final Call call, final IOException e) {
                progress.dismiss();
                Log.e(TAG, "Failed purchase request", e);

                activity.showDialog("Failed to send the request: " + e.getMessage(), "OK");

                transaction.close();
            }

            @Override
            public void onResponse(Call call, Response response) throws IOException {
                progress.dismiss();

                String responseBody = response.body().string();
                Log.d(TAG, "Purchase response: " + responseBody);

                final PurchaseResponse purchaseResponse = new Gson().fromJson(responseBody, PurchaseResponse.class);
                lastTransactionId = purchaseResponse.getThreeDSServerTransID();
                Log.d(TAG, "threeDSServerTransID: " + lastTransactionId);

                if (purchaseResponse.getTransStatus() != null
                        && purchaseResponse.getTransStatus().equals(CHALLENGE_TRANS_STATUS)) {

                    ChallengeParameters challengeParameters = new ChallengeParameters();
                    challengeParameters.setAcsRefNumber(purchaseResponse.getAcsReferenceNumber());
                    challengeParameters.set3DSServerTransactionID(purchaseResponse.getThreeDSServerTransID());
                    challengeParameters.setAcsSignedContent(purchaseResponse.getAcsSignedContent());
                    challengeParameters.setAcsTransactionID(purchaseResponse.getAcsTransID());

                    transaction.doChallenge(activity, challengeParameters, new ChallengeStatusReceiver() {
                        @Override
                        public void completed(final CompletionEvent completionEvent) {
                            if (lastTransactionId != null) {
                                Log.d(TAG, "Transaction ID: " + lastTransactionId);
                            }

                            String message = String.format("The completed callback called, threeDSServerTransID=%s, status=%s", lastTransactionId, completionEvent.getTransactionStatus());
                            activity.showDialog(message, "OK");
                            transaction.close();
                        }

                        @Override
                        public void cancelled() {
                            if (lastTransactionId != null) {
                                Log.d(TAG, "Transaction ID: " + lastTransactionId);
                            }

                            String message = String.format("The cancelled callback called, threeDSServerTransID=%s", lastTransactionId);
                            activity.showDialog(message, "OK");
                            transaction.close();
                        }

                        @Override
                        public void timedout() {
                            if (lastTransactionId != null) {
                                Log.d(TAG, "Transaction ID: " + lastTransactionId);
                            }

                            String message = String.format("The timedout callback called, threeDSServerTransID=%s", lastTransactionId);
                            activity.showDialog(message, "OK");
                            transaction.close();
                        }

                        @Override
                        public void protocolError(ProtocolErrorEvent protocolErrorEvent) {
                            if (lastTransactionId != null) {
                                Log.d(TAG, "Transaction ID: " + lastTransactionId);
                            }

                            Log.d(TAG, String.format("The runtimeError callback called, error message: %s", protocolErrorEvent.getErrorMessage()));
                            String message = String.format("The protocolError callback called, threeDSServerTransID=%s", lastTransactionId);
                            activity.showDialog(message, "OK");
                            transaction.close();
                        }

                        @Override
                        public void runtimeError(RuntimeErrorEvent runtimeErrorEvent) {
                            if (lastTransactionId != null) {
                                Log.d(TAG, "Transaction ID: " + lastTransactionId);
                            }

                            Log.d(TAG, String.format("The runtimeError callback called, error code: %s, error message: %s", runtimeErrorEvent.getErrorCode(), runtimeErrorEvent.getErrorMessage()));
                            String message = String.format("The runtimeError callback called, threeDSServerTransID=%s", lastTransactionId);
                            activity.showDialog(message, "OK");
                            transaction.close();
                        }
                    }, 30);
                } else {
                    transaction.close();

                    if (lastTransactionId != null) {
                        Log.d(TAG, "Transaction ID: " + lastTransactionId);
                    }
                    String message = String.format("The purchase completed, threeDSServerTransID=%s, status=%s", lastTransactionId, purchaseResponse.getTransStatus());
                    activity.showDialog(message, "OK");
                }
            }
        });
    }

    private void getSupportedVersionsByDsRequest(PurchaseData purchaseData, Callback callback) {
        Map<String, Object> supportedVersionsRequest = new HashMap<>();
        supportedVersionsRequest.put("cardID", purchaseData.getAccountNumber());
        supportedVersionsRequest.put("sdkProtocolVersions", new ArrayList<String>(){{
            add("2.1.0");
            add("2.2.0");
        }});

        Request request = new Request.Builder()
                .url(purchaseData.getBackendUrl() + SUPPORTED_VERSIONS_PATH)
                .header("Authorization", String.format("Bearer %s:%s", purchaseData.getEmail(), purchaseData.getPassword()))
                .post(RequestBody.create(JSON, new Gson().toJson(supportedVersionsRequest)))
                .build();

        client.newCall(request).enqueue(callback);
    }

    private ConfigParameters getConfigParameters() {
        ConfigParameters result = new ConfigParameters();

        result.addParam(USDK_CONFIG_PARAMETER_GROUP, LICENSE_PARAMETER_KEY, licenseKey);

        BitmapFactory.Options options = new BitmapFactory.Options();
        options.inPreferredConfig = Bitmap.Config.RGB_565;
        options.inSampleSize = 2;

        DirectoryServer msigniaDsRsa = new DirectoryServer(MSIGNIA_DIRECTORY_SERVER_RSA_ID, MSIGNIA_DIRECTORY_SERVER_RSA_KEY_ID,
                MSIGNIA_DIRECTORY_SERVER_RSA_KEY, MSIGNIA_DIRECTORY_SERVER_CERT, "Mastercard");
        msigniaDsRsa.setDsLogo(BitmapFactory.decodeResource(activity.getResources(), R.drawable.ds_logo_mastercard, options));

        DirectoryServer msigniaDsEc = new DirectoryServer(MSIGNIA_DIRECTORY_SERVER_EC_ID, MSIGNIA_DIRECTORY_SERVER_EC_KEY_ID,
                MSIGNIA_DIRECTORY_SERVER_EC_KEY, MSIGNIA_DIRECTORY_SERVER_CERT, "Carte Bancaire");
        msigniaDsEc.setDsLogo(BitmapFactory.decodeResource(activity.getResources(), R.drawable.ds_logo_cb, options));

        DirectoryServer demoMcDs = new DirectoryServer("DEMO-MC-RSA-DS_REF_NUMBER", MSIGNIA_DIRECTORY_SERVER_RSA_KEY_ID,
                MSIGNIA_DIRECTORY_SERVER_RSA_KEY, MSIGNIA_DIRECTORY_SERVER_CERT, "Mastercard", R.drawable.ds_logo_mastercard);
        demoMcDs.setDsLogo(BitmapFactory.decodeResource(activity.getResources(), R.drawable.ds_logo_mastercard, options));

        DirectoryServer demoVisaDs = new DirectoryServer("DEMO-VISA-RSA-DS_REF_NUMBER", MSIGNIA_DIRECTORY_SERVER_RSA_KEY_ID,
                MSIGNIA_DIRECTORY_SERVER_RSA_KEY, MSIGNIA_DIRECTORY_SERVER_CERT, "Visa");
        demoVisaDs.setDsLogo(BitmapFactory.decodeResource(activity.getResources(), R.drawable.ds_logo_visa, options));

        DirectoryServer demoDiscoverDs = new DirectoryServer("DEMO-DISCOVER-RSA-DS_REF_NUMBER", MSIGNIA_DIRECTORY_SERVER_RSA_KEY_ID,
                MSIGNIA_DIRECTORY_SERVER_RSA_KEY, MSIGNIA_DIRECTORY_SERVER_CERT, "Discover");
        demoDiscoverDs.setDsLogo(BitmapFactory.decodeResource(activity.getResources(), R.drawable.ds_logo_discover, options));

        DirectoryServer demoAmexDs = new DirectoryServer("DEMO-AMEX-RSA-DS_REF_NUMBER", MSIGNIA_DIRECTORY_SERVER_RSA_KEY_ID,
                MSIGNIA_DIRECTORY_SERVER_RSA_KEY, MSIGNIA_DIRECTORY_SERVER_CERT, "American Express", R.drawable.ds_logo_amex);

        DirectoryServer demoCbDs = new DirectoryServer("DEMO-CB-RSA-DS_REF_NUMBER", MSIGNIA_DIRECTORY_SERVER_RSA_KEY_ID,
                MSIGNIA_DIRECTORY_SERVER_RSA_KEY, MSIGNIA_DIRECTORY_SERVER_CERT, "Carte Bancaire");
        demoCbDs.setDsLogo(BitmapFactory.decodeResource(activity.getResources(), R.drawable.ds_logo_cb, options));

        DirectoryServer demoJcbDs = new DirectoryServer("DEMO-JCB-RSA-DS_REF_NUMBER", MSIGNIA_DIRECTORY_SERVER_RSA_KEY_ID,
                MSIGNIA_DIRECTORY_SERVER_RSA_KEY, MSIGNIA_DIRECTORY_SERVER_CERT, "JCB");
        demoJcbDs.setDsLogo(BitmapFactory.decodeResource(activity.getResources(), R.drawable.ds_logo_jcb, options));

        result.addDirectoryServer(msigniaDsRsa);
        result.addDirectoryServer(msigniaDsEc);
        result.addDirectoryServer(demoMcDs);
        result.addDirectoryServer(demoVisaDs);
        result.addDirectoryServer(demoDiscoverDs);
        result.addDirectoryServer(demoAmexDs);
        result.addDirectoryServer(demoCbDs);
        result.addDirectoryServer(demoJcbDs);

        return result;
    }
}
