package com.android.sample.usdk;

import android.util.Log;

import com.android.sample.MainActivity;
import com.android.sample.models.PurchaseData;

import org.emvco.threeds.core.AuthCallback;
import org.emvco.threeds.core.AuthSpec;
import org.emvco.threeds.core.PurchaseInfo;
import org.emvco.threeds.core.TransStatus;
import org.json.JSONException;
import org.json.JSONObject;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.HashMap;

public class SimplifiedInterfaceUsdkManager extends UsdkManager {

    private static final String TAG = SimplifiedInterfaceUsdkManager.class.getSimpleName();

    public SimplifiedInterfaceUsdkManager(MainActivity activity, String licenseKey) {
        super(activity, licenseKey);
    }

    @Override
    public void purchase(final PurchaseData purchaseData) {
        Log.d(TAG, "Purchase using simplified interface");
        AuthSpec authSpec = new AuthSpec();
        authSpec.setCurrentActivity(activity);
        authSpec.setCardId(purchaseData.getAccountNumber());
        authSpec.setAuthorizationHeaders(new HashMap<String, String>() {{
            put("Authorization", String.format("Bearer %s:%s", purchaseData.getEmail(), purchaseData.getPassword()));
        }});
        authSpec.setSupportedVersionsURL(purchaseData.getBackendUrl() + SUPPORTED_VERSIONS_PATH);
        authSpec.setAuthURL(purchaseData.getBackendUrl()  + AUTH_PATH);
        authSpec.setPaymentAuthentication(true);
        authSpec.setPurchaseInfo(new PurchaseInfo() {
            @Override
            public JSONObject toJson() {
                JSONObject json = new JSONObject();
                try {
                    json.put("email", purchaseData.getEmail());
                    json.put("cardID", purchaseData.getAccountNumber());
                    json.put("purchaseAmount", "2495");
                    json.put("purchaseCurrency", "840");
                    json.put("purchaseExponent", "2");
                    json.put("purchaseDate", new SimpleDateFormat("yyyyMMddHHmmss").format(Calendar.getInstance().getTime()));
                } catch (JSONException e) {
                    Log.e(TAG, "Can not create purchase info", e);
                }

                return json;
            }
        });

        threeDS2Service.authenticate(authSpec, new AuthCallback() {
            @Override
            public void authenticated(String transId, TransStatus transStatus) {
                lastTransactionId = transId;
                Log.d(TAG, "Transaction ID: " + transId);
                String message = String.format("The authenticated callback called, threeDSServerTransID=%s, status=%s", transId, transStatus.name());
                Log.d(TAG, message);
                activity.showDialog(message, "OK");
            }

            @Override
            public void notAuthenticated(String transId, TransStatus transStatus) {
                lastTransactionId = transId;

                String message;
                if (transStatus == null) {
                    message = String.format("The notAuthenticated callback called, threeDSServerTransID=%s", transId);
                } else {
                    message = String.format("The notAuthenticated callback called, threeDSServerTransID=%s, status=%s", transId, transStatus.name());
                    Log.d(TAG, "Transaction ID: " + transId);
                }

                Log.d(TAG, message);
                activity.showDialog(message, "OK");
            }

            @Override
            public void decoupledAuthBeingPerformed(String transId, TransStatus transStatus) {
                lastTransactionId = transId;

                Log.d(TAG, "Transaction ID: " + transId);
                String message = String.format("The decoupledAuthBeingPerformed callback called, threeDSServerTransID=%s, status=%s", transId, transStatus.name());
                Log.d(TAG, message);
                activity.showDialog(message, "OK");
            }

            @Override
            public void cancelled(String transId) {
                lastTransactionId = transId;

                Log.d(TAG, "Transaction ID: " + transId);
                String message = String.format("The cancelled callback called, threeDSServerTransID=%s", transId);
                Log.d(TAG, message);
                activity.showDialog(message, "OK");
            }

            @Override
            public void error(String transId, Exception e) {
                lastTransactionId = transId;

                if (transId != null) {
                    Log.d(TAG, "Transaction ID: " + transId);
                }
                String message = String.format("The error callback called, threeDSServerTransID=%s, error=%s", transId, e.getMessage());
                Log.d(TAG, message);
                activity.showDialog(message, "OK");
            }
        });
    }
}
