package com.android.sample.usdk;

import android.app.ProgressDialog;
import androidx.core.os.ConfigurationCompat;

import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.util.Log;

import com.android.sample.MainActivity;
import com.android.sample.R;
import com.android.sample.models.DirectoryServerItem;
import com.android.sample.models.PurchaseData;
import com.google.gson.Gson;
import com.usdk.android.UsdkThreeDS2ServiceImpl;

import org.emvco.threeds.core.AuthenticationRequestParameters;
import org.emvco.threeds.core.ConfigParameters;
import org.emvco.threeds.core.DirectoryServer;
import org.emvco.threeds.core.InitCallback;
import org.emvco.threeds.core.InitSpec;
import org.emvco.threeds.core.ThreeDS2Service;
import org.emvco.threeds.core.Warning;
import org.emvco.threeds.core.ui.UiCustomization;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.HashMap;
import java.util.Map;

public abstract class UsdkManager {

    private static final String TAG = UsdkManager.class.getSimpleName();

    public static final String APP_DEVICE_CHANEL_VALUE = "01";
    public static final String PAYMENT_MESSAGE_CATEGORY_VALUE = "01";

    public static final String SUPPORTED_VERSIONS_PATH = "/api/v6/supported-versions";
    public static final String AUTH_PATH = "/api/v6/auth";

    public ThreeDS2Service threeDS2Service;
    public static String lastTransactionId;
    protected MainActivity activity;

    protected String licenseKey;

    protected UsdkManager(MainActivity activity, String licenseKey) {
        this.activity = activity;
        this.licenseKey = licenseKey;
        threeDS2Service = new UsdkThreeDS2ServiceImpl();
    }

    public abstract void purchase(final PurchaseData purchaseData);

    public void initializeUsdk(final InitializeUsdkResultCallback callback) {
        final ProgressDialog progress = new ProgressDialog(activity);
        progress.setMessage(activity.getString(R.string.progress_usdk_init));
        progress.setIndeterminate(true);
        progress.setCancelable(false);
        progress.show();

        InitSpec initSpec = new InitSpec();
        initSpec.setApplicationContext(activity);
        initSpec.setLicenseKey(licenseKey);
        initSpec.setConfigParameters(getConfigParameters());
        initSpec.setLocale(ConfigurationCompat.getLocales(activity.getResources().getConfiguration()).get(0).toString());

        UiCustomization uiCustomization = new UiCustomization();
        uiCustomization.setTheme(R.style.AppTheme);

//        LabelCustomization labelCustomization = new LabelCustomization();
//        labelCustomization.setHeadingTextFontSize(26);
//        labelCustomization.setTextColor("#007987");
//        uiCustomization.setLabelCustomization(labelCustomization);
//
//        ButtonCustomization buttonCustomization = new ButtonCustomization();
//        buttonCustomization.setBackgroundColor("#55bab7");
//        uiCustomization.setButtonCustomization(buttonCustomization, ButtonType.SUBMIT);
        initSpec.setUiCustomization(uiCustomization);

        threeDS2Service.initialize(initSpec, new InitCallback() {
            @Override
            public void onSuccess() {
                progress.dismiss();

                Log.d(TAG, "3DS SDK initialized successfully");
                for (Warning warning : threeDS2Service.getWarnings()) {
                    Log.w(TAG, String.format("Detected 3DS warning with ID %s and message: %s", warning.getID(), warning.getMessage()));
                }

                callback.onSuccess();
            }

            @Override
            public void onError(Exception e) {
                progress.dismiss();

                Log.e(TAG, "uSDK initialization failed");

                callback.onError(e.getMessage());
            }
        });
    }

    private ConfigParameters getConfigParameters() {
        ConfigParameters result = new ConfigParameters();

        BitmapFactory.Options options = new BitmapFactory.Options();
        options.inPreferredConfig = Bitmap.Config.RGB_565;
        options.inSampleSize = 2;

        DirectoryServer msigniaDsRsa = new DirectoryServer(MSIGNIA_DIRECTORY_SERVER_RSA_ID, MSIGNIA_DIRECTORY_SERVER_RSA_KEY_ID,
                MSIGNIA_DIRECTORY_SERVER_RSA_KEY, MSIGNIA_DIRECTORY_SERVER_CERT, "Mastercard");
        msigniaDsRsa.setDsLogo(BitmapFactory.decodeResource(activity.getResources(), R.drawable.ds_logo_mastercard, options));

        DirectoryServer msigniaDsEc = new DirectoryServer(MSIGNIA_DIRECTORY_SERVER_EC_ID, MSIGNIA_DIRECTORY_SERVER_EC_KEY_ID,
                MSIGNIA_DIRECTORY_SERVER_EC_KEY, MSIGNIA_DIRECTORY_SERVER_CERT, "Carte Bancaire");
        msigniaDsEc.setDsLogo(BitmapFactory.decodeResource(activity.getResources(), R.drawable.ds_logo_cb, options));

        DirectoryServer demoMcDs = new DirectoryServer("DEMO-MC-RSA-DS_REF_NUMBER", MSIGNIA_DIRECTORY_SERVER_RSA_KEY_ID,
                MSIGNIA_DIRECTORY_SERVER_RSA_KEY, MSIGNIA_DIRECTORY_SERVER_CERT, "Mastercard");
        demoMcDs.setDsLogo(BitmapFactory.decodeResource(activity.getResources(), R.drawable.ds_logo_mastercard, options));

        DirectoryServer demoVisaDs = new DirectoryServer("DEMO-VISA-RSA-DS_REF_NUMBER", MSIGNIA_DIRECTORY_SERVER_RSA_KEY_ID,
                MSIGNIA_DIRECTORY_SERVER_RSA_KEY, MSIGNIA_DIRECTORY_SERVER_CERT, "Visa");
        demoVisaDs.setDsLogo(BitmapFactory.decodeResource(activity.getResources(), R.drawable.ds_logo_visa, options));

        DirectoryServer demoDiscoverDs = new DirectoryServer("DEMO-DISCOVER-RSA-DS_REF_NUMBER", MSIGNIA_DIRECTORY_SERVER_RSA_KEY_ID,
                MSIGNIA_DIRECTORY_SERVER_RSA_KEY, MSIGNIA_DIRECTORY_SERVER_CERT, "Discover");
        demoDiscoverDs.setDsLogo(BitmapFactory.decodeResource(activity.getResources(), R.drawable.ds_logo_discover, options));

        DirectoryServer demoAmexDs = new DirectoryServer("DEMO-AMEX-RSA-DS_REF_NUMBER", MSIGNIA_DIRECTORY_SERVER_RSA_KEY_ID,
                MSIGNIA_DIRECTORY_SERVER_RSA_KEY, MSIGNIA_DIRECTORY_SERVER_CERT, "American Express", R.drawable.ds_logo_amex);

        DirectoryServer demoCbDs = new DirectoryServer("DEMO-CB-RSA-DS_REF_NUMBER", MSIGNIA_DIRECTORY_SERVER_RSA_KEY_ID,
                MSIGNIA_DIRECTORY_SERVER_RSA_KEY, MSIGNIA_DIRECTORY_SERVER_CERT, "Carte Bancaire");
        demoCbDs.setDsLogo(BitmapFactory.decodeResource(activity.getResources(), R.drawable.ds_logo_cb, options));

        DirectoryServer demoJcbDs = new DirectoryServer("DEMO-JCB-RSA-DS_REF_NUMBER", MSIGNIA_DIRECTORY_SERVER_RSA_KEY_ID,
                MSIGNIA_DIRECTORY_SERVER_RSA_KEY, MSIGNIA_DIRECTORY_SERVER_CERT, "JCB");
        demoJcbDs.setDsLogo(BitmapFactory.decodeResource(activity.getResources(), R.drawable.ds_logo_jcb, options));

        result.addDirectoryServer(msigniaDsRsa);
        result.addDirectoryServer(msigniaDsEc);
        result.addDirectoryServer(demoMcDs);
        result.addDirectoryServer(demoVisaDs);
        result.addDirectoryServer(demoDiscoverDs);
        result.addDirectoryServer(demoAmexDs);
        result.addDirectoryServer(demoCbDs);
        result.addDirectoryServer(demoJcbDs);

        return result;
    }

    protected Map<String, Object> generateAuthenticationRequest(PurchaseData purchaseData, AuthenticationRequestParameters authParams, DirectoryServerItem directoryServerItem, Map<String, Object> extraAuthParams, boolean dualBrandedCard) {
        Map<String, Object> result = new HashMap<>();
        Map<String, Object> authRequestParams = new HashMap<>();
        if (dualBrandedCard) {
            authRequestParams.put("dsIdentifier", directoryServerItem.getDsIdentifier());
        }
        authRequestParams.put("supportedVersionsThreeDSServerTransID", directoryServerItem.getThreeDSServerTransID());
        authRequestParams.put("messageVersion", authParams.getMessageVersion());
        authRequestParams.put("messageCategory", PAYMENT_MESSAGE_CATEGORY_VALUE);
        authRequestParams.put("deviceChannel", APP_DEVICE_CHANEL_VALUE);
        authRequestParams.put("deviceRenderOptions", new Gson().fromJson("{\n" +
                "\"sdkInterface\":\"03\",\n" +
                "\"sdkUiType\":[\"01\",\"02\",\"03\",\"04\",\"05\"]\n" +
                "}", Map.class));
        authRequestParams.put("deviceData", authParams.getDeviceData());
        authRequestParams.put("sdkTransactionID", authParams.getSDKTransactionID());
        authRequestParams.put("sdkAppID", authParams.getSDKAppID());
        authRequestParams.put("sdkReferenceNumber", authParams.getSDKReferenceNumber());
        authRequestParams.put("sdkEphemeralPublicKey", new Gson().fromJson(authParams.getSDKEphemeralPublicKey(), Map.class));
        authRequestParams.put("sdkMaxTimeout", "10");
        if (extraAuthParams != null) {
            authRequestParams.put("extraAuthParams", extraAuthParams);
        }

        Map<String, Object> purchaseInfo = new HashMap<>();
        purchaseInfo.put("email", purchaseData.getEmail());
        purchaseInfo.put("cardID", purchaseData.getAccountNumber());
        purchaseInfo.put("purchaseAmount", "2495");
        purchaseInfo.put("purchaseCurrency", "840");
        purchaseInfo.put("purchaseExponent", "2");
        purchaseInfo.put("purchaseDate", new SimpleDateFormat("yyyyMMddHHmmss").format(Calendar.getInstance().getTime()));

        result.put("purchaseInfo", purchaseInfo);
        result.put("authenticationRequestParameters", authRequestParams);

        Log.d(TAG, "Auth request: " + result.toString());
        return result;
    }

    public static final String MSIGNIA_DIRECTORY_SERVER_RSA_ID = "MSIGNIA_MOCK_DS_RSA";
    public static final String MSIGNIA_DIRECTORY_SERVER_RSA_KEY = "{\"kty\":\"RSA\",\"e\":\"AQAB\",\"kid\":\"de7655be-3d6f-4802-9d68-c36a9d23e71c\",\"n\":\"4Uko-Z2ESFV5IUXunK08ouSziFz_OfuRR9H4woBdMbAeyAq0-V5o36tCtiNVYgF2Oi1jBgmEIxIMH9y-W_Mn0OoiwzV7eQj4tc0q5RM9RzYEMoRCK-0YU4nFzRlDSFQOIPEkEWihPHujyY32qBDyMsl0ctlq5EN-F77u9wCvoxU9OLM3dlAE42dNrbrHWOnpeWg41kk63AUowZlA7QE52Tgd_hyaHAy4oQtgp1_-l-SPIAKIoAF-5Ibiss4Y1Wah1RSvDz69UMgHoqc_cvWG9kTPMtkFoIwW8ta2jhGzvhoNf2X1o2excBLKlWT3hIwGfnw8oYRDlINTZIuTGLc0cw\"}";
    public static final String MSIGNIA_DIRECTORY_SERVER_RSA_KEY_ID = "de7655be-3d6f-4802-9d68-c36a9d23e71c";

    public static final String MSIGNIA_DIRECTORY_SERVER_EC_ID = "MSIGNIA_MOCK_DS_EC";
    public static final String MSIGNIA_DIRECTORY_SERVER_EC_KEY = "{\"kty\":\"EC\",\"crv\":\"P-256\",\"x\":\"7YTOG_nT8NtzC7sgQDEEsbkFX6VEXdzXgpoU0FD7gRY\",\"y\":\"Th1ej6HF0K9yVg3DM0K6zQrxERBSoX597LqadlIq77o\"}";
    public static final String MSIGNIA_DIRECTORY_SERVER_EC_KEY_ID = "";

    public static final String MSIGNIA_DIRECTORY_SERVER_CERT = "\"-----BEGIN CERTIFICATE-----\n" +
            "MIID4zCCAsugAwIBAgIBATANBgkqhkiG9w0BAQsFADB7MQswCQYDVQQGEwJVUzEL\n" +
            "MAkGA1UECBMCVE4xEDAOBgNVBAoTB01TSUdOSUExDDAKBgNVBAsTA1JORDEXMBUG\n" +
            "A1UEAxMOTVNJR05JQSBSTkQgQ0ExJjAkBgkqhkiG9w0BCQEWF3BhdmxvLmx5c292\n" +
            "QG1zaWduaWEuY29tMB4XDTE5MTExMzA3MzYwMFoXDTI5MTExMzA3MzYwMFowezEL\n" +
            "MAkGA1UEBhMCVVMxCzAJBgNVBAgTAlROMRAwDgYDVQQKEwdNU0lHTklBMQwwCgYD\n" +
            "VQQLEwNSTkQxFzAVBgNVBAMTDk1TSUdOSUEgUk5EIENBMSYwJAYJKoZIhvcNAQkB\n" +
            "FhdwYXZsby5seXNvdkBtc2lnbmlhLmNvbTCCASIwDQYJKoZIhvcNAQEBBQADggEP\n" +
            "ADCCAQoCggEBAPSn+PdwUVXYq5IQN+vrcxM3WfR5Xpgb3OpIRkka863duH0j3Uh/\n" +
            "poaFxnp/j0dGbefYaKaXGZSFX/E3FWaAFGKHV2Lbt4uxYkJx4Fa2gXsBxqdOd4Li\n" +
            "aqSRSZXPtYMqoMbCOQj4lqtthBNe5BDX6/1HDJELLo8sxnIyfwxgbome3s9e6Rpz\n" +
            "p+M9XgOtfyuSzPhzY0bWeBrHDzPcKkxQgKZeywlgApCPPfoYEvBDfzGbNns4yA+v\n" +
            "sqYGDcctplQceUG6EFN68qbmy3CzV8zQMHFkHqIJ/l1V5RlGT9b5QhNVhu4MJo//\n" +
            "W8LaA6vdyyqPnS7Hbvp6JklkMVxJJhWl0t8CAwEAAaNyMHAwDwYDVR0TAQH/BAUw\n" +
            "AwEB/zAdBgNVHQ4EFgQU443mM7cesLlNiq01jfVfyJ7Rh1gwCwYDVR0PBAQDAgEG\n" +
            "MBEGCWCGSAGG+EIBAQQEAwIABzAeBglghkgBhvhCAQ0EERYPeGNhIGNlcnRpZmlj\n" +
            "YXRlMA0GCSqGSIb3DQEBCwUAA4IBAQBOY7ETJ0MyrQ44MYNbkGI7fIij5oxM0In0\n" +
            "Z4ZT5NCA1p3fJP/PV8oxONE+7ix1GlCw8RR59L2VQGkg4I2vSj0DHqpuPf3u86Fm\n" +
            "gZtkhcF41dlWSic1B5oaA610iglITgu/gyZrC+r6RHGIKfyLX+u8YGc6WfI8g/ED\n" +
            "aVQcnq/PZAEeZtBGiZdYxnIQhIOxV04nFmtOjSyyMt5NXMHFLHRnewSI6tojcFm2\n" +
            "lm6WMZQXFqGyvkxjmPQXFFBCupAAPIkm0lIIvZqtvLOOqmU7et4Oy5pihyaHjg67\n" +
            "cdmlv/p990e7YYfzdQSKezWrdwXW4EAGBTyuAZ67a5XCOUi17JZU\n" +
            "-----END CERTIFICATE-----\"";
}
